using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing.Design;
using Dapfor.Net.Data;

namespace Dapfor.Net.Demo.Examples.DataBinding.ConditionalDataBinding
{
    public class Basket : INotifyPropertyChanged
    {
        #region Fields

        private readonly DateTime _creationDate;
        private readonly string _id;
        private readonly BindingList<Order> _orders = new BindingList<Order>();
        private DateTime _lastChangeTimeStamp;

        #endregion

        public Basket(string id)
        {
            _id = id;
            _creationDate = _lastChangeTimeStamp = DateTime.Now;
        }

        #region Public properties

        public IList<Order> Orders
        {
            get { return _orders; }
        }

        [Field("TimeStamp")]
        public DateTime CreationDate
        {
            get { return _creationDate; }
        }

        public DateTime LastChangeTimeStamp
        {
            get { return _lastChangeTimeStamp; }
        }

        public string Id
        {
            get { return _id; }
        }

        [Editor(typeof(BasketStatusEditor), typeof(UITypeEditor))]
        public Status Status
        {
            get
            {
                Status status = Status.Created;
                foreach (Order order in _orders)
                {
                    switch (order.Status)
                    {
                        case Status.Cancelled:
                            if (!Equals(status, Status.Active))
                            {
                                status = Status.Cancelled;
                            }
                            break;

                        case Status.Active:
                            status = Status.Active;
                            break;
                    }
                }
                return status;
            }
        }

        #endregion

        #region Public methods

        public void AddOrder(Order order)
        {
            if (order == null)
            {
                throw new ArgumentNullException("order");
            }

            _orders.Add(order);
            order.PropertyChanged += OnOrderStatusChanged;
        }

        #endregion

        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;

        #endregion

        #region Private methods

        private void OnOrderStatusChanged(object sender, PropertyChangedEventArgs e)
        {
            UpdateChangeTimeStamp();
            switch (e.PropertyName)
            {
                case "Status":
                    FirePropertyChanged("Status");
                    break;
            }
        }

        private void UpdateChangeTimeStamp()
        {
            _lastChangeTimeStamp = DateTime.Now;
            FirePropertyChanged("LastChangeTimeStamp");
        }

        private void FirePropertyChanged(string id)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(id));
            }
        }

        #endregion

    }
}