using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing.Design;
using Dapfor.Net.Data;
using Dapfor.Net.Formats;

namespace Dapfor.Net.Demo.Examples.DataBinding.ConditionalDataBinding
{
    public class Order : INotifyPropertyChanged
    {
        #region Fields

        private readonly Basket _basket;
        private readonly DateTime _creationDate;
        private readonly BindingList<Deal> _deals = new BindingList<Deal>();
        private readonly string _id;
        private readonly string _product;
        private DateTime _lastChangeTimeStamp;
        private double _price;
        private long _quantity;
        private Status _status;
        private Way _way;
        

        #endregion

        public Order(string id, Way way, double price, long quantity, string product, Basket basket)
        {
            _id = id;
            _product = product;
            _way = way;
            _price = price;
            _quantity = quantity;
            _basket = basket;
            _creationDate = _lastChangeTimeStamp = DateTime.Now;
        }

        #region Public properties

        public IList<Deal> Deals
        {
            get { return _deals; }
        }

        public string Id
        {
            get { return _id; }
        }

        public string Product
        {
            get { return _product; }
        }

        public Basket Basket
        {
            get { return _basket; }
        }

        [Field("TimeStamp")]
        public DateTime CreationDate
        {
            get { return _creationDate; }
        }

        public DateTime LastChangeTimeStamp
        {
            get { return _lastChangeTimeStamp; }
        }

        public Way Way
        {
            get { return _way; }
            set
            {
                UpdateChangeTimeStamp();
                if (!Equals(_way, value))
                {
                    _way = value;
                    FirePropertyChanged("Way");
                }
            }
        }

        [DoubleFormat(3)]
        public double Price
        {
            get { return _price; }
            set
            {
                UpdateChangeTimeStamp();
                if (_price != value)
                {
                    _price = value;
                    FirePropertyChanged("Price");
                }
            }
        }

        public long Quantity
        {
            get { return _quantity; }
            set
            {
                UpdateChangeTimeStamp();
                if (_quantity != value)
                {
                    _quantity = value;
                    FirePropertyChanged("Quantity");
                }
            }
        }

        [Editor(typeof(OrderStatusEditor), typeof(UITypeEditor))]
        public Status Status
        {
            get { return _status; }
            set
            {
                if (!Equals(_status, value))
                {
                    _status = value;
                    FirePropertyChanged("Status");
                }
            }
        }

        #endregion

        #region Public methods

        public void AddDeal(Deal deal)
        {
            if (deal == null)
            {
                throw new ArgumentNullException("deal");
            }
            UpdateChangeTimeStamp();
            _deals.Add(deal);
        }

        #endregion

        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;

        #endregion

        #region Private methods

        private void UpdateChangeTimeStamp()
        {
            _lastChangeTimeStamp = DateTime.Now;
            FirePropertyChanged("LastChangeTimeStamp");
        }

        private void FirePropertyChanged(string id)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(id));
            }
        }

        #endregion
    }
}