using System;
using System.Drawing;
using System.Windows.Forms;
using Dapfor.Net.Demo.Internal;
using Dapfor.Net.Ui;

namespace Dapfor.Net.Demo.Examples.Features.CustomEditors
{
    [Example("Custom editors", "Features")]
    public partial class CustomEditorsDemoForm : UserControl
    {
        private readonly Random _random = new Random();

        public CustomEditorsDemoForm()
        {
            InitializeComponent();

            //Set a checkBox editor for the first column
            grid.Headers[0]["rating"].Editor = new RatingEditor();
            grid.Headers[0]["color"].Editor = new CustomColorEditor();
            grid.Headers[0]["trackbar"].Editor = new CustomTrackBarEditor(10);
            
            PopulateGrid();

            //Initialize controls
            cbHotFrame.Checked = grid.EditInPlace.HotFrameEnabled;
            btnHotFrameColor.Color = grid.EditInPlace.HotFrameColor;
            cbEnabled.Checked = grid.EditInPlace.Enabled;
            cbShowDropButtons.Checked = grid.EditInPlace.AlwaysShowDropButtons;
            cbAdjust.Checked = grid.EditInPlace.AdjustAfterEdit;
        }

        private void PopulateGrid()
        {
            //Add 20 custom objects to the grid
            for (int i = 0; i < 20; ++i)
            {
                grid.Rows.Add(new DataObject(_random));
            }
        }

        private void OnHotFrameEnabled(object sender, EventArgs e)
        {
            grid.EditInPlace.HotFrameEnabled = cbHotFrame.Checked;
        }

        private void OnHotFrameColorChanged(object sender, EventArgs e)
        {
            grid.EditInPlace.HotFrameColor = btnHotFrameColor.Color;
        }

        private void OnEnabledChanged(object sender, EventArgs e)
        {
            grid.EditInPlace.Enabled = cbEnabled.Checked;
        }

        private void OnShowDropButtonsChanged(object sender, EventArgs e)
        {
            grid.EditInPlace.AlwaysShowDropButtons = cbShowDropButtons.Checked;
        }

        private void OnAdjustChanged(object sender, EventArgs e)
        {
            grid.EditInPlace.AdjustAfterEdit = cbAdjust.Checked;
        }

        private void OnPaintGroupRow(object sender, PaintGroupRowEventArgs e)
        {
            if (e.Column.Id == "rating")
            {
                e.Text = "Rating:";
                e.PaintAll();
                e.Handled = true;

                int width = (int)Math.Ceiling(e.Graphics.MeasureString(e.Text, e.Font).Width);

                //Draw the stars over the cell
                Rectangle bounds = e.Bounds;
                bounds.Y += (bounds.Height - Properties.Resources.star_grey.Height) / 2;
                bounds.Height = Properties.Resources.star_grey.Height;
                bounds.X += width;
                bounds.Width = Properties.Resources.star_grey.Width;

                for (int i = 0; i < 5; i++)
                {
                    //Select image
                    int curRating = (int)e.Value;
                    Image image = i > curRating ? Properties.Resources.star_grey : Properties.Resources.star_yellow;
                    e.Graphics.DrawImage(image, bounds);
                    bounds.X += bounds.Width;
                }
            }
            else if (e.Column.Id == "bullet")
            {
                switch ((int)e.Value)
                {
                    case 0:
                        e.Image = Properties.Resources.bullet_ball_glass_grey;
                        break;
                    case 1:
                        e.Image = Properties.Resources.bullet_ball_glass_green;
                        break;
                    case 2:
                        e.Image = Properties.Resources.bullet_ball_glass_blue;
                        break;
                    case 3:
                        e.Image = Properties.Resources.bullet_ball_glass_yellow;
                        break;
                    case 4:
                        e.Image = Properties.Resources.bullet_ball_glass_red;
                        break;
                }
            }
        }
    }
}
