using System;
using System.Drawing;
using System.Windows.Forms;
using Dapfor.Net.Demo.Internal;
using Dapfor.Net.Ui;

namespace Dapfor.Net.Demo.Examples.Features.FixedColumns
{
    [Example("Fixed columns", "Features")]
    public partial class FixedColumnsDemoForm : UserControl
    {
        #region Fields

        private readonly Random random = new Random();
        
        #endregion

        public FixedColumnsDemoForm()
        {
            //Initialize the grid
            InitializeComponent();

            //Populate the grid
            for (int i = 0; i < 20; ++i)
            {
                DataObject rootObject = new DataObject(random);
                Row root = grid.Rows.Add(rootObject);
                for (int j = 0; j < 10; ++j)
                {
                    root.Add(new DataObject(random));
                }
            }

            grid.Rows.ExpandAll();
        }

        private void OnPaintCell(object sender, PaintCellEventArgs e)
        {
            if (e.Cell.Row != null && 
                e.Cell.Column != null)
            {
                if(e.Cell.Column.VisibleIndex < e.Cell.Column.Header.FixedColumns.Count)
                {
                    e.Appearance.BackColor = PaintHelper.AlphaBlend(e.Appearance.BackColor, Color.FromArgb(40, Color.Gray));
                    e.Appearance.GradientEndBackColor = PaintHelper.AlphaBlend(e.Appearance.GradientEndBackColor, Color.FromArgb(40, Color.Gray));
                }
            }
        }

        private void OnTopHeaderVisibleChanged(object sender, EventArgs e)
        {
            //Set visibility of the top header
            grid.Headers[0].Visible = cbTopHeaderVisible.Checked;
        }

        private void OnTopHeaderFixedColumnsChanged(object sender, EventArgs e)
        {
            //Set count of fixed rows of the top header
            grid.Headers[0].FixedColumns.Count = (int) udTopFixedColumns.Value;
        }

        private void OnChildHeaderVisibleChanged(object sender, EventArgs e)
        {
            //Set visibility of the child header
            grid.Headers[1].Visible = cbChildHeaderVisible.Checked;
        }

        private void OnChildHeaderFixedColumnsChanged(object sender, EventArgs e)
        {
            //Set count of fixed rows of the child header
            grid.Headers[1].FixedColumns.Count = (int)udChildFixedColumns.Value;
        }

        private void OnScrollTypeChanged(object sender, EventArgs e)
        {
            ColumnScrollType scrollType = ColumnScrollType.Optimized;
            if(rbNormal.Checked)
            {
                scrollType = ColumnScrollType.Normal;
            }

            grid.Headers[0].ScrollType = scrollType;
            grid.Headers[1].ScrollType = scrollType;
        }

    }
}