using System;
using System.ComponentModel;
using System.Drawing;
using Dapfor.Net.Data;
using Dapfor.Net.Ui;

namespace Dapfor.Net.Demo.Examples.Features.StandardEditors
{
    /// <summary>
    /// The objects of this class will be inserted into the <see cref="Grid"/>. 
    /// </summary>
    public class DataObject : INotifyPropertyChanged
    {
        #region Fields 

        private bool boolValue1;
        private bool boolValue2;
        private Color color1;
        private ContentAlignment contentAlignement;
        private Font font;
        private int intValue1;
        private DateTime dateTime;


        #endregion

        #region Public properties

        public DataObject(Random r)
        {
            //Populate fields with random values
            if (r != null)
            {
                boolValue1 = r.Next()%2 == 0;
                boolValue2 = r.Next()%2 == 0;
                
                ContentAlignment[] v = (ContentAlignment[]) Enum.GetValues(typeof(ContentAlignment));
                contentAlignement = v[r.Next()%v.Length];

                intValue1 = r.Next();
                
                color1 = Color.FromArgb(r.Next()%0x100, r.Next()%0x100, r.Next()%0x100);
                font = SystemFonts.CaptionFont;

                dateTime = DateTime.Today;
            }
        }

        [Field("Id1")]
        public bool BoolValue1
        {
            get { return boolValue1; }
            set
            {
                if (boolValue1 != value)
                {
                    boolValue1 = value;
                    if (PropertyChanged != null)
                    {
                        PropertyChanged(this, new PropertyChangedEventArgs("Id1"));
                    }
                }
            }
        }

        [Field("Id2")]
        public bool BoolValue2
        {
            get { return boolValue2; }
            set
            {
                if (boolValue2 != value)
                {
                    boolValue2 = value;
                    if (PropertyChanged != null)
                    {
                        PropertyChanged(this, new PropertyChangedEventArgs("Id2"));
                    }
                }
            }
        }


        [Field("Id3")]
        public ContentAlignment ContentAlignement
        {
            get { return contentAlignement; }
            set
            {
                if (!Equals(contentAlignement, value))
                {
                    contentAlignement = value;
                    if (PropertyChanged != null)
                    {
                        PropertyChanged(this, new PropertyChangedEventArgs("Id3"));
                    }
                }
            }
        }

        [Field("Id4")]
        public int IntValue1
        {
            get { return intValue1; }
            set
            {
                if (intValue1 != value)
                {
                    intValue1 = value;
                    if (PropertyChanged != null)
                    {
                        PropertyChanged(this, new PropertyChangedEventArgs("Id4"));
                    }
                }
            }
        }

        [Field("Id5")]
        public Color Color
        {
            get { return color1; }
            set
            {
                if (!Equals(color1, value))
                {
                    color1 = value;
                    if (PropertyChanged != null)
                    {
                        PropertyChanged(this, new PropertyChangedEventArgs("Id5"));
                    }
                }
            }
        }

        [Field("Id6")]
        public Font Font
        {
            get { return font; }
            set
            {
                if (!Equals(font, value))
                {
                    font = value;
                    if (PropertyChanged != null)
                    {
                        PropertyChanged(this, new PropertyChangedEventArgs("Id6"));
                    }
                }
            }
        }

        [Field("Id7")]
        public DateTime DateTime
        {
            get { return dateTime; }
            set 
            {
                if (!Equals(dateTime, value))
                {
                    dateTime = value; 
                    if (PropertyChanged != null)
                    {
                        PropertyChanged(this, new PropertyChangedEventArgs("Id7"));
                    }
                }
            }
        }

        #endregion

        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;

        #endregion
    }
}