using System;
using System.Windows.Forms;
using Dapfor.Net.Demo.Internal;
using Dapfor.Net.Ui;

namespace Dapfor.Net.Demo.Examples.GridBasics.FirstLook
{
    [Example("First look", ".Net Grid basics")]
    public partial class GridBasicsForm : UserControl
    {
        #region Constructor

        /// <summary>
        /// Initializes a new instance of the <see cref="GridBasicsForm"/> class.
        /// </summary>
        public GridBasicsForm()
        {
            //Initialize all controls
            InitializeComponent();

            //Populate the grid
            PopulateDataObjects();

            //Update the controls with the current properties
            UpdateControls();
        }

        #endregion

        #region Private methods

        private void PopulateDataObjects()
        {
            //Remove all previous rows and headers
            //grid.Headers.AutoGenerate = true;

            Random random = new Random();

            //Populate with some data
            for (int i = 0; i < 20; ++i)
            {
                //Populate with top-level items
                DataBinding.Populating.RootClass rootObject = new DataBinding.Populating.RootClass(random.NextDouble(),
                                                                           random.Next(5),
                                                                           "Item " + i,
                                                                           random.Next()%2 == 0 ? DataBinding.Populating.RootClass.EnumExample.Internal : DataBinding.Populating.RootClass.EnumExample.External);
                //Add to the grid
                Row rootRow = grid.Rows.Add(rootObject);
                rootRow.Expanded = true;

                //Add ten children to each root
                for (int j = 0; j < 10; ++j)
                {
                    DataBinding.Populating.ChildClass childObject = new DataBinding.Populating.ChildClass(DateTime.Now + new TimeSpan(1000000000000*(random.Next()%1000)),
                                                                                  random.Next(5),
                                                                                  new TimeSpan(random.Next()));
                    //Add as a child
                    rootRow.Add(childObject);
                }
            }

            //Adjust columns
            grid.Headers[0].AdjustColumns(true, false);
            grid.Headers[1].AdjustColumns(true, false);
        }

        private void UpdateControls()
        {
            Header rootHeader = grid.Headers[0];
            Header childHeader = grid.Headers[1];

            cbTopHeader.Checked = rootHeader.Visible;
            cbChildHeader.Checked = childHeader.Visible;

            //Hierarchy
            cbHierarchicalLines.Checked = grid.Hierarchy.EnableHierarchicalLines;
            cbExpansionButton.Checked = !Equals(ExpansionButtonBehaviour.HideAlways, grid.Hierarchy.ButtonBehaviour);

            //Set even and odd rows colors
            btnRootEvenColor.Color = rootHeader.Appearance.EvenRows.BackColor;
            btnRootOddColor.Color = rootHeader.Appearance.OddRows.BackColor;
            btnChildEvenColor.Color = childHeader.Appearance.EvenRows.BackColor;
            btnChildOddColor.Color = childHeader.Appearance.OddRows.BackColor;

            //Set top and child header colors
            btnRootHeaderStartColor.Color = rootHeader.Appearance.ColumnPanel.BackColor;
            btnRootHeaderEndColor.Color = rootHeader.Appearance.ColumnPanel.GradientEndBackColor;
            btnChildHeaderStartColor.Color = childHeader.Appearance.ColumnPanel.BackColor;
            btnChildHeaderEndColor.Color = childHeader.Appearance.ColumnPanel.GradientEndBackColor;
        }

        #endregion

        #region Private event handlers

        private void OnChildHeaderEndColorColorChanged(object sender, EventArgs e)
        {
            grid.Headers[1].Appearance.ColumnPanel.GradientEndBackColor = btnChildHeaderEndColor.Color;
            grid.Headers[1].Appearance.GroupPanel = grid.Headers[1].Appearance.ColumnPanel;
        }

        private void OnChildHeaderStartColorColorChanged(object sender, EventArgs e)
        {
            grid.Headers[1].Appearance.ColumnPanel.BackColor = btnChildHeaderStartColor.Color;
            grid.Headers[1].Appearance.GroupPanel = grid.Headers[1].Appearance.ColumnPanel;
        }

        private void OnRootHeaderEndColorColorChanged(object sender, EventArgs e)
        {
            grid.Headers[0].Appearance.ColumnPanel.GradientEndBackColor = btnRootHeaderEndColor.Color;
            grid.Headers[0].Appearance.GroupPanel = grid.Headers[0].Appearance.ColumnPanel;
        }

        private void OnRootHeaderStartColorColorChanged(object sender, EventArgs e)
        {
            grid.Headers[0].Appearance.ColumnPanel.BackColor = btnRootHeaderStartColor.Color;
            grid.Headers[0].Appearance.GroupPanel = grid.Headers[0].Appearance.ColumnPanel;
        }

        private void OnChildOddColorColorChanged(object sender, EventArgs e)
        {
            grid.Headers[1].Appearance.OddRows.BackColor = btnChildOddColor.Color;
        }

        private void OnChildEvenColorColorChanged(object sender, EventArgs e)
        {
            grid.Headers[1].Appearance.EvenRows.BackColor = btnChildEvenColor.Color;
        }

        private void OnRootOddColorColorChanged(object sender, EventArgs e)
        {
            grid.Headers[0].Appearance.OddRows.BackColor = btnRootOddColor.Color;
        }

        private void OnRootEvenColorColorChanged(object sender, EventArgs e)
        {
            grid.Headers[0].Appearance.EvenRows.BackColor = btnRootEvenColor.Color;
        }

        private void OnTopHeaderCheckedChanged(object sender, EventArgs e)
        {
            grid.Headers[0].Visible = cbTopHeader.Checked;
        }

        private void OnChildHeaderCheckedChanged(object sender, EventArgs e)
        {
            grid.Headers[1].Visible = cbChildHeader.Checked;
        }

        private void OnRowSelectorCheckedChanged(object sender, EventArgs e)
        {
            grid.RowSelector.Visible = cbRowSelector.Checked;
        }

        private void OnHierarchicalLinesCheckedChanged(object sender, EventArgs e)
        {
            grid.Hierarchy.EnableHierarchicalLines = cbHierarchicalLines.Checked;
        }

        private void OnExpansionButtonCheckedChanged(object sender, EventArgs e)
        {
            grid.Hierarchy.ButtonBehaviour = cbExpansionButton.Checked
                                                 ? ExpansionButtonBehaviour.HideWhenNoChildren
                                                 : ExpansionButtonBehaviour.HideAlways;
        }

        private void OnGroupPanelCheckedChanged(object sender, EventArgs e)
        {
            grid.Headers[0].GroupingEnabled = cbGroupPanel.Checked;
        }

        #endregion
    }
}
